@@ -15,8 +15,9 @@ class ClientAuthorization(Client):
         data: Union[Dict, bytes, None] = None,
         params: Optional[Dict] = None,
         headers: Optional[Dict] = None,
+        retry: bool = True,
     ) -> Client.Response:
         headers = headers or {}
         headers["Authorization"] = self.token_provider.get_token()
         headers["user-agent"] = self.token_provider.user_agent
-        return super()._request(path, data, params, headers)
+        return super()._request(path, data, params, headers, retry)
