@@ -6,6 +6,7 @@ from typing import TYPE_CHECKING, Dict, Iterator, Optional, Union
 
 import requests
 from requests.adapters import HTTPAdapter
+from urllib3 import Retry
 
 from odp.tabular_v2.util import Iter2Reader
 from odp.util.util import size2human
@@ -13,6 +14,20 @@ from odp.util.util import size2human
 if TYPE_CHECKING:
     from odp.tabular_v2.client.table import Table
 
+read_retry_strategy = Retry(
+    total=8,
+    status_forcelist=[429, 502, 503, 504],
+    allowed_methods=["HEAD", "GET", "OPTIONS", "POST"],
+    backoff_factor=1,  # exponential sleep (1s, 2s, 4s…)
+)
+
+write_retry_strategy = Retry(
+    total=8,
+    status_forcelist=[429],
+    allowed_methods=["HEAD", "GET", "OPTIONS", "POST"],
+    backoff_factor=2,
+)
+
 
 class Client:
     def __init__(self, base_url: str):
@@ -109,9 +124,15 @@ class Client:
         data: Union[Dict, bytes, Iterator[bytes], io.IOBase, None] = None,
         params: Optional[Dict] = None,
         headers: Optional[Dict] = None,
+        retry: bool = True,  # if it is safe to retry the request on failure
     ) -> Response:
+        if retry:
+            retry_strategy = read_retry_strategy  # read operations are safe to retry in many cases
+        else:
+            retry_strategy = write_retry_strategy  # more restrictive
+
         with requests.Session() as s:
-            adapter = HTTPAdapter(pool_connections=4, pool_maxsize=4, max_retries=8)
+            adapter = HTTPAdapter(pool_connections=4, pool_maxsize=4, max_retries=retry_strategy)
             s.mount("http://", adapter)
             s.mount("https://", adapter)
             if isinstance(data, dict):
