@@ -33,6 +33,7 @@ def _request_to_new_client(
     data: Union[Dict, bytes, Iterator[bytes], io.IOBase, None] = None,
     params: Optional[Dict] = None,
     headers: Optional[Dict] = None,
+    retry: bool = True,  # if it is safe to retry the request on failure
 ) -> old.Client.Response:
     req = requests.Request(
         method="POST",
@@ -41,13 +42,15 @@ def _request_to_new_client(
         headers=headers,
         data=data,
     )
-    res = cli._request(req)
+    res = cli._request(req, retry)
     return old.Client.Response(res)
 
 
 class Table:
     def __init__(self, impl, table_id: str):
         self._id = table_id
+        self.max_insert_size = 50_000_000  # maximum number of bytes for a single insert chunk
+        self.max_insert_time = 30.0  # min time before flushing on a new request
 
         # unify new and old client _request() method
         import odp.new_client as new
@@ -89,6 +92,19 @@ class Table:
         assert empty[0].num_rows == 0
         return empty[0].schema
 
+    def truncate(self):
+        """
+        Remove all data from the table, but keep the schema.
+        """
+        self._impl(
+            path="/api/table/v2/truncate",
+            params={
+                "table_id": self._id,
+            },
+            retry=False,
+        ).all()
+        logging.info("truncated %s", self._id)
+
     def alter(self, schema: pa.Schema, from_names: dict = {}):
         """
         perform a schema change, re-ingesting all the data in the table with the new schema
@@ -110,6 +126,7 @@ class Table:
                 "table_id": self._id,
             },
             data=buf.getvalue(),  # send the schema using pa.ipc
+            retry=False,
         )
         return res.json()
 
@@ -125,6 +142,7 @@ class Table:
                 params={
                     "table_id": self._id,
                 },
+                retry=False,
             ).json()
             logging.debug("dropped %s: %s", self._id, res)
         except FileNotFoundError:
@@ -168,6 +186,7 @@ class Table:
                 "table_id": self._id,
             },
             data=buf.getvalue(),
+            retry=False,
         ).json()
 
     def aggregate(
@@ -218,7 +237,7 @@ class Table:
                 raise ValueError(f"unknown aggregation type: {a_type}")
 
         total: Union[pd.DataFrame, None] = None
-        for b in self._select(type="aggregate", by=group_by or "'TOTAL'", inner_query=filter, aggr=aggr, vars=vars):
+        for b in self._select(typ="aggregate", by=group_by or "'TOTAL'", inner_query=filter, aggr=aggr, vars=vars):
             df: pd.DataFrame = b.to_pandas()
             # logging.warning("PARTIAL:\n%s", df)
             if total is None:
@@ -259,14 +278,14 @@ class Table:
         cols: Optional[List[str]] = None,
         vars: Union[dict, list, None] = None,
         stream_ttl: float = 30.0,
-        type: str = "select",
+        typ: str = "select",
         tx_id: str = "",
     ) -> "Cursor":
         def scanner(scanner_cursor: str) -> Iterator[pa.RecordBatch]:
             logging.debug("selecting cursor=%s, filter=%s", scanner_cursor, filter)
             for b in self._select(
                 tx=tx_id,
-                type=type,
+                typ=typ,
                 inner_query=filter,
                 cols=cols,
                 vars=vars,
@@ -328,6 +347,7 @@ class Table:
             params={
                 "table_id": self._id,
             },
+            retry=False,
         ).json()
         from odp.tabular_v2.client.table_tx import Transaction
 
@@ -355,6 +375,7 @@ class Table:
                     "table_id": self._id,
                     "tx_id": self._tx._id,
                 },
+                retry=False,
             )
         self._tx = None
 
@@ -362,7 +383,7 @@ class Table:
     def _select(
         self,
         tx: str = "",
-        type: str = "select",
+        typ: str = "select",
         inner_query: str = "",
         aggr: Optional[dict] = None,
         cols: Optional[List[str]] = None,
@@ -374,7 +395,7 @@ class Table:
         # t0 = time.perf_counter()
         while cursor is not None:
             res = self._impl(
-                path="/api/table/v2/sdk/" + type,
+                path="/api/table/v2/sdk/" + typ,
                 params={
                     "table_id": self._id,
                     "tx_id": tx,
@@ -388,6 +409,7 @@ class Table:
                     "vars": vars_to_json(vars),
                     "timeout": timeout,
                 },
+                retry=typ in {"select", "aggregate"},
             )
             cursor = None
             reader = res.reader()
@@ -424,4 +446,5 @@ class Table:
                 "tx_id": tx,
             },
             data=buf.getvalue(),
+            retry=False,
         ).json()
